local ADDON, Package = ...

local Enum = Package.Enum
local System = Package.System

local Module = {}
local Registry = {}

local agent = CreateFrame("Frame")

function Module.run()
  agent:RegisterEvent("PLAYER_LOGIN")
  agent:RegisterEvent("PLAYER_ENTERING_WORLD")
  agent:RegisterEvent("ADDON_LOADED")
  agent:SetScript("OnEvent", function (self, event, ...) Registry[event](self, ...) end)
end

function Module.load_storage()
  local schema = {
    version = 2.0,
    minimap_icon_drawer = true,
    use_item_quality_color_border = true,
    use_arrow_keys_in_chat = true,
    increase_chat_history = true,
    unclamp_chat_frame = true,
    use_sticky_raid_warning_in_chat = true,
    use_class_color_in_chat = true,
    use_max_distance = true,
    use_max_zoom_factor = true,
    posture_after_action_attempted = true,
    repair_when_interact_merchant = true,
    sell_junk_when_interact_merchant = true,
    rework_time_alert = true,
    search_profession = true,
    indicate_accurate_buff_time = true,
    indicate_item_id_and_level_on_tooltip = true,
    indicate_item_vendor_price_on_tooltip = true,
    indicate_target_level_and_class_color_in_trade = true,
    copy_url_in_chat = true,
    log_chat = true,
    prevent_unit_combat_feedback = false,
    prevent_ui_error_message_during_combat = false,
  }

  local schema_per_character = {
    version = 2.0,
    chat_log = {
      ChatFrame1 = {},
      ChatFrame3 = {},
      ChatFrame4 = {},
      ChatFrame5 = {},
      ChatFrame6 = {},
      ChatFrame7 = {},
      ChatFrame8 = {},
      ChatFrame9 = {},
      ChatFrame10 = {}
    }
  }

  MedusaPreludeStorage = Enum.merge(MedusaPreludeStorage or {}, schema, false)
  MedusaPreludeStoragePerCharacter = Enum.merge(MedusaPreludeStoragePerCharacter or {}, schema_per_character, false)
end

function Registry.PLAYER_LOGIN()
  local controlled_options = {
    use_class_color_in_chat = Package.CVar.configure_chat_class_color_override,
    use_max_distance = Package.CVar.configure_nameplate_max_distance
  }

  local uncontrolled_options = {
    minimap_icon_drawer = Package.Minimap.configure_icon_drawer,
    use_item_quality_color_border = Package.UseItemQualityColorBorder.enable,
    use_arrow_keys_in_chat = Package.Chat.configure_alt_arrow_key_mode,
    increase_chat_history = Package.Chat.configure_max_lines,
    unclamp_chat_frame = Package.Chat.configure_clamp_rect_insets,
    use_sticky_raid_warning_in_chat = Package.Chat.configure_raid_warning,
    posture_after_action_attempted = Package.Action.configure_to_stand_or_dismount,
    repair_when_interact_merchant = Package.Merchant.configure_to_repair_equips,
    sell_junk_when_interact_merchant = Package.Merchant.configure_to_sell_junk,
    rework_time_alert = Package.TimeAlert.configure,
    search_profession = Package.SearchProfession.enable,
    indicate_accurate_buff_time = Package.Buff.configure_aura_duration,
    indicate_item_id_and_level_on_tooltip = Package.GameTooltip.configure_id_and_level,
    indicate_item_vendor_price_on_tooltip = Package.GameTooltip.configure_sell_price,
    indicate_target_level_and_class_color_in_trade = Package.Trade.configure_recipient_name_with_class_and_level,
    copy_url_in_chat = Package.Chat.configure_to_copy_url_to_clipboard,
    log_chat = Package.Chat.configure_logger,
    prevent_unit_combat_feedback = Package.Combat.configure_unit_combat,
    prevent_ui_error_message_during_combat = Package.Combat.configure_ui_error_message
  }

  for key, value in pairs(MedusaPreludeStorage) do
    if controlled_options[key] then
      controlled_options[key](value)
    elseif uncontrolled_options[key] and value then
      uncontrolled_options[key]()
    end
  end
end

function Registry.PLAYER_ENTERING_WORLD()
  Package.CVar.configure_camera_distance_max_zoom_factor(MedusaPreludeStorage.use_max_zoom_factor)
end

function Registry.ADDON_LOADED(self, ...)
  if ADDON == ... then
    Module.load_storage()
    System.prime()
  end
end

Module.run()

SLASH_MedusaPrelude1 = "/medusa_prelude"
SLASH_MedusaPrelude2 = "/mp"
SlashCmdList[ADDON] = function ()
  if not MedusaPreludeConfig then
    Package.Config.run()
  end

  MedusaPreludeConfig:SetShown(not MedusaPreludeConfig:IsShown())
end

SLASH_MedusaPreludeSendChatMessage1 = "/scm"
SlashCmdList[ADDON .. "SendChatMessage"] = function (args)
  if args and args:trim() and args ~= "" then
    local channel, message = string.split(" ", args, 2)

    if channel and channel:match("^[1-9]$") and message and message:len() > 0 then
      message = message:gsub("{([a-z]+):([0-9]+)}", function (type, number)
        local id = tonumber(number)

        if id > 0 and id < 0x7FFFFFFF then
          local link = nil

          if type == "item" then
            _, link = GetItemInfo(id)
          elseif type == "spell" then
            link = GetSpellLink(id)
          end

          if link then
            return link
          end

          return ""
        end
      end)

      SendChatMessage(message, "CHANNEL", DEFAULT_CHAT_FRAME.editBox.languageID, channel)
    end
  end
end
