local ADDON, Package = ...

local L = Package.System.L
local Widget = Package.Widget

local Module = {}

local PREFERENCES = {
  {
    "minimap_icon_drawer",
    "use_item_quality_color_border",
    "use_max_distance",
    "use_max_zoom_factor",

    "posture_after_action_attempted",
    "repair_when_interact_merchant",
    "sell_junk_when_interact_merchant",
    "rework_time_alert",
    "search_profession",

    "indicate_accurate_buff_time",
    "indicate_item_id_and_level_on_tooltip",
    "indicate_item_vendor_price_on_tooltip",
    "indicate_target_level_and_class_color_in_trade"
  },
  {
    "use_arrow_keys_in_chat",
    "increase_chat_history",
    "unclamp_chat_frame",
    "use_sticky_raid_warning_in_chat",
    "use_class_color_in_chat",
    "copy_url_in_chat",
    "log_chat"
  },
  {
    "prevent_unit_combat_feedback",
    "prevent_ui_error_message_during_combat"
  }
}

function Module.run()
  local storage = MedusaPreludeStorage
  local window = Widget.create_window({size = {320, 400}, name = ADDON .. "Config"}, UIParent)

  local tabs = Widget.create_tabs(window)

  for id, prefs in pairs(PREFERENCES) do
    local relative = nil
    local parent = tabs[id].panel

    for at = 1, #prefs do
      local pref = prefs[at]

      local checkbox = Widget.create_checkbox({text = L[pref]}, parent)
      checkbox:SetID(at)
      checkbox:SetSize(24, 24)
      checkbox:SetScript("OnShow", function (self)
        self:SetChecked(storage[pref])
      end)
      checkbox:SetScript("OnClick", function (self)
        storage[pref] = self:GetChecked()
      end)
      checkbox.text:SetWidth(window:GetWidth() - 48)
      checkbox.text:SetJustifyH("LEFT")

      if not relative then
        checkbox:SetPoint("TOPLEFT", 8, -8)
      else
        checkbox:SetPoint("TOP", relative, "BOTTOM", 0, -4)
      end

      relative = checkbox
    end
  end

  local button = Widget.create_button({text = L["reload_button"]}, window)
  button:SetPoint("BOTTOM", 0, 8)
  button:SetScript("OnClick", function (self) C_UI.Reload() end)

  window:SetPoint("CENTER")

  tabs[1].button:GetScript("OnClick")(tabs[1].button)

  table.insert(UISpecialFrames, MedusaPreludeConfig:GetName())
end

Package.Config = Module
