local ADDON, Package = ...

local Kernel = Package.Kernel

local Module = {}

local TOOLTIP_TEXT_RIGHTS = {
  GameTooltip = GameTooltipTextRight1,
  ShoppingTooltip1 = ShoppingTooltip1TextRight2,
  ShoppingTooltip2 = ShoppingTooltip2TextRight2,
  ItemRefTooltip = ItemRefTooltipTextRight1
}

function Module.configure_id_and_level()
  local f = function (self)
    local item_name, item_link = self:GetItem()

    if item_name == "" and CraftFrame and CraftFrame:IsShown() then
      local container = GetMouseFocus()

      item_link = GetCraftReagentItemLink(GetCraftSelectionIndex(), container:GetID())
    end

    if item_link then
      local item_id = GetItemInfoFromHyperlink(item_link)
      local _, _, _, item_level = GetItemInfo(item_link)

      if item_id and item_level then
        local text_right = TOOLTIP_TEXT_RIGHTS[self:GetName()]
        local item_id = GRAY_FONT_COLOR:WrapTextInColorCode(item_id)
        local item_level = NORMAL_FONT_COLOR:WrapTextInColorCode(ITEM_LEVEL:format(item_level))

        text_right:SetFormattedText("%s %s", item_id, item_level)
        text_right:Show()
      end
    end
  end

  GameTooltip:HookScript("OnTooltipSetItem", f)
  ItemRefTooltip:HookScript("OnTooltipSetItem", f)
  ShoppingTooltip1:HookScript("OnTooltipSetItem", f)
  ShoppingTooltip2:HookScript("OnTooltipSetItem", f)

  hooksecurefunc(GameTooltip, "SetHyperlink", f)
  hooksecurefunc(ItemRefTooltip, "SetHyperlink", f)
end

function Module.configure_sell_price()
  local f = function (self)
    local item_name, item_link = self:GetItem()

    if item_name == "" and CraftFrame and CraftFrame:IsShown() then
      local container = GetMouseFocus()

      item_link = GetCraftReagentItemLink(GetCraftSelectionIndex(), container:GetID())
    end

    if item_link then
      local _, _, _, _, _, _, _, _, _, _, item_sell_price = GetItemInfo(item_link)
      local container = GetMouseFocus()
      local item_count = 1

      if Kernel.is_table(container) and Kernel.is_number(container.count) then
        item_count = container.count
      end

      if item_count and item_count > 0 and item_sell_price and item_sell_price > 0 and self.shownMoneyFrames == nil then
        SetTooltipMoney(self, item_count * item_sell_price, "STATIC")
      end
    end
  end

  GameTooltip:HookScript("OnTooltipSetItem", f)
  ItemRefTooltip:HookScript("OnTooltipSetItem", f)
  ShoppingTooltip1:HookScript("OnTooltipSetItem", f)
  ShoppingTooltip2:HookScript("OnTooltipSetItem", f)

  hooksecurefunc(GameTooltip, "SetHyperlink", f)
  hooksecurefunc(ItemRefTooltip, "SetHyperlink", f)
end

Package.GameTooltip = Module
