local ADDON, Package = ...

local Enum = Package.Enum
local L = Package.System.L

local Module = {}

function Module.purge_keyboard_focus()
  local frame = GetCurrentKeyBoardFocus()
  if frame and frame.ClearFocus then
    frame:ClearFocus()
  end
end

function Module.create_window(optional_attrs, parent)
  local default_attrs = {
    text = "",
    size = {128, 128},
    name = nil
  }
  local attrs = Enum.merge(default_attrs, optional_attrs or {})

  local frame = CreateFrame("Frame", attrs.name, parent)
  frame:Hide()
  frame:EnableKeyboard(true)
  frame:EnableMouse(true)
  frame:EnableMouseWheel(true)
  frame:SetToplevel(true)
  frame:SetMovable(true)
  frame:SetFrameStrata("DIALOG")
  frame:SetSize(unpack(attrs.size))
  frame:SetScript("OnMouseUp", function (self) Module.purge_keyboard_focus() end)

  local background = frame:CreateTexture(nil, "BACKGROUND")
  background:SetAllPoints(frame)
  background:SetTexture(131071)

  local border_topleft = frame:CreateTexture(nil, "ARTWORK")
  border_topleft:SetAtlas("battlefieldminimap-border-topleft", true)
  border_topleft:SetPoint("TOPLEFT", -10, 10)

  local border_topright = frame:CreateTexture(nil, "ARTWORK")
  border_topright:SetAtlas("battlefieldminimap-border-topright", true)
  border_topright:SetPoint("TOPRIGHT", 8, 10)

  local border_bottomleft = frame:CreateTexture(nil, "ARTWORK")
  border_bottomleft:SetAtlas("battlefieldminimap-border-bottomleft", true)
  border_bottomleft:SetPoint("BOTTOMLEFT", -10, -9)

  local border_bottomright = frame:CreateTexture(nil, "ARTWORK")
  border_bottomright:SetAtlas("battlefieldminimap-border-bottomright", true)
  border_bottomright:SetPoint("BOTTOMRIGHT", 8, -9)

  local border_top = frame:CreateTexture(nil, "ARTWORK")
  border_top:SetAtlas("battlefieldminimap-border-top")
  border_top:SetPoint("TOPLEFT", border_topleft, "TOPRIGHT")
  border_top:SetPoint("BOTTOMRIGHT", border_topright, "BOTTOMLEFT")

  local border_bottom = frame:CreateTexture(nil, "ARTWORK")
  border_bottom:SetAtlas("battlefieldminimap-border-bottom")
  border_bottom:SetPoint("TOPLEFT", border_bottomleft, "TOPRIGHT")
  border_bottom:SetPoint("BOTTOMRIGHT", border_bottomright, "BOTTOMLEFT")

  local border_left = frame:CreateTexture(nil, "ARTWORK")
  border_left:SetAtlas("battlefieldminimap-border-left")
  border_left:SetPoint("TOPLEFT", border_topleft, "BOTTOMLEFT")
  border_left:SetPoint("BOTTOMRIGHT", border_bottomleft, "TOPRIGHT")

  local border_right = frame:CreateTexture(nil, "ARTWORK")
  border_right:SetAtlas("battlefieldminimap-border-right")
  border_right:SetPoint("TOPLEFT", border_topright, "BOTTOMLEFT")
  border_right:SetPoint("BOTTOMRIGHT", border_bottomright, "TOPRIGHT")

  local close_button_border = frame:CreateTexture(nil, "BORDER")
  close_button_border:SetTexture(131073)
  close_button_border:SetSize(32, 32)
  close_button_border:SetPoint("TOPRIGHT", 1, 2)

  local close_button = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
  close_button:SetPoint("TOPRIGHT", 5, 6)

  frame.close_button_border = close_button_border
  frame.close_button = close_button

  return frame
end

function Module.create_tab_button(optional_attrs, parent)
  local default_attrs = {
    text = ""
  }
  local attrs = Enum.merge(default_attrs, optional_attrs or {})

  local frame = CreateFrame("Button", nil, parent)
  frame:SetSize(64, 32)

  local left = frame:CreateTexture(nil, "BACKGROUND")
  left:SetTexture(130942)
  left:SetTexCoord(0.0, 0.25, 0.0, 1.0)
  left:SetSize(16, 32)
  left:SetPoint("TOPLEFT")

  local middle = frame:CreateTexture(nil, "BACKGROUND")
  middle:SetTexture(130942)
  middle:SetTexCoord(0.25, 0.75, 0.0, 1.0)
  middle:SetSize(44, 32)
  middle:SetPoint("LEFT", left, "RIGHT")

  local right = frame:CreateTexture(nil, "BACKGROUND")
  right:SetTexture(130942)
  right:SetTexCoord(0.75, 1.0, 0.0, 1.0)
  right:SetSize(16, 32)
  right:SetPoint("LEFT", middle, "RIGHT")

  local highlight = frame:CreateTexture(nil, "BACKGROUND")
  highlight:SetTexture(136580)
  highlight:SetPoint("LEFT", left, 0, -7)
  highlight:SetPoint("RIGHT", right, 0, -7)
  highlight:SetBlendMode("ADD")

  local pushed = frame:CreateTexture(nil, "BACKGROUND")
  pushed:SetTexture(375515)
  pushed:SetPoint("LEFT", left, 0, -7)
  pushed:SetPoint("RIGHT", right, 0, -7)
  pushed:SetBlendMode("ADD")

  local font_string = frame:CreateFontString()
  font_string:SetPoint("LEFT", left, 8, -6)
  font_string:SetPoint("RIGHT", right, -8, -6)
  font_string:SetJustifyH("CENTER")

  frame:SetHighlightTexture(highlight)
  frame:SetPushedTexture(pushed)
  frame:SetPushedTextOffset(0, 0)
  frame:SetNormalFontObject("GameFontNormalSmall")
  frame:SetFontString(font_string)
  frame:SetText(attrs.text)

  return frame
end

function Module.create_tab_panel(optional_attrs, parent)
  local default_attrs = {}
  local attrs = Enum.merge(default_attrs, optional_attrs or {})

  local frame = CreateFrame("Frame", nil, parent)
  frame:Hide()

  return frame
end

function Module.create_button(optional_attrs, parent)
  local default_attrs = {
    text = "",
    size = {80, 24},
    tooltip = nil
  }
  local attrs = Enum.merge(default_attrs, optional_attrs or {})

  local frame = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate")
  frame:SetText(attrs.text)
  frame:SetSize(unpack(attrs.size))
  frame.tooltipText = attrs.tooltip

  return frame
end

function Module.create_checkbox(optional_attrs, parent)
  local default_attrs = {
    text = "",
    checked = false
  }
  local attrs = Enum.merge(default_attrs, optional_attrs or {})

  local frame = CreateFrame("CheckButton", nil, parent, "UICheckButtonTemplate")
  frame:SetChecked(attrs.checked)
  frame.text:SetText(attrs.text)

  return frame
end

function Module.create_searchbox(optional_attrs, parent)
  local default_attrs = {
    name = nil
  }
  local attrs = Enum.merge(default_attrs, optional_attrs or {})
  local frame = CreateFrame("EditBox", attrs.name, parent, "SearchBoxTemplate")

  return frame
end

function Module.create_tabs(parent)
  local tabs = {}
  local tabs_attrs = {
    {name = "general", text = L["general"]},
    {name = "chat", text = L["chat"]},
    {name = "combat", text = L["combat"]}
  }

  for at = 1, #tabs_attrs do
    local tab = {}

    tab.name = tabs_attrs[at].name

    tab.button = Module.create_tab_button({text = tabs_attrs[at].text}, parent)
    tab.button:SetID(at)
    tab.button:SetPoint("TOPLEFT", (at - 1) * 74, 36)
    tab.button:SetScript("OnDragStart", function (self) self:GetParent():StartMoving() end)
    tab.button:SetScript("OnDragStop", function (self) self:GetParent():StopMovingOrSizing() end)
    tab.button:RegisterForDrag("LeftButton")

    tab.panel = Module.create_tab_panel(nil, parent)
    tab.panel:SetID(at)
    tab.panel:SetAllPoints(parent)

    tabs[at] = tab
  end

  for at = 1, #tabs do
    tabs[at].button:SetScript("OnClick", function (self)
      local id = self:GetID()

      for at = 1, #tabs do
        tabs[at].button:SetButtonState("NORMAL", false)
        tabs[at].panel:Hide()

        tabs[id].button:SetButtonState("PUSHED", true)
        tabs[id].panel:Show()
      end
    end)
  end

  return tabs
end

Package.Widget = Module
